import sys
import os
import os.path
import stat
import zipfile
import modulefinder
from distutils.sysconfig import get_python_lib

"""
The Distributor class makes a python executable zip archive out of a script.

The pubic method is `package(filename)`.

The zipfile will be created in the directory specified in the class attribute `working_directory`.

Any paths included listed in the `excluded paths` attribute will be excluded from the package.  By default
the standard library is excluded.


"""




class Distributor(object):
    working_directory = 'zipdist'
    included_path  = sys.path.extend(['.',])
    
    def __init__(self, exclude_path_list=None, 
                    hashbang=False,
                    extension='pyz',
                    verbose=0):
        self.excluded_paths = [get_python_lib(standard_lib=True), ]
        if exclude_path_list:
            for excl in exclude_path_list:
                self.excluded_paths.append(excl)
        
        if hashbang is False: 
            self.hashbang = '#!/usr/bin/env python3'
        else:
            self.hashbang = hashbang
    
        self.extension = extension
        self.verbose   = verbose
    
    def find_libraries(self, filename):
        """
Find all modules uses by a given script.  Exclude any paths listed in `excluded_paths`.

Returns (modulename, module) for each of them.
        """
        M = modulefinder.ModuleFinder()
        M.run_script(filename)
        for k in M.modules.keys():
            #print(k)
            include_this = True
            if M.modules[k].__path__:
                for p in M.modules[k].__path__:
                    for excluded_path in self.excluded_paths:
                        if p.startswith(excluded_path):
                            include_this = False
                            if self.verbose > 1:
                                print(" -- excluding: ", p)
                            break
                if include_this:
                    if self.verbose > 0:
                        for pt in M.modules[k].__path__:
                            print(" ++ including: ", pt)
                    #print((k, M.modules[k]))
                    yield (k, M.modules[k])
                
    
    def package(self, filename):
        if not os.path.exists(filename):
            raise FileNotFoundError
        working_dir = self.working_directory
        working_zip_file = os.path.join(working_dir, os.path.splitext(filename)[0] + '.pyz' + '.zip')
        if self.extension:
            output_path = os.path.join(working_dir, os.path.splitext(filename)[0] + '.' + self.extension)
        else:
            output_path = os.path.join(working_dir, os.path.splitext(filename)[0])
        print("Creating: " + output_path)
        if not os.path.exists(working_dir):
            os.mkdir(working_dir)
        
        if os.path.exists(output_path):
            os.remove(output_path)
            
        if os.path.exists(working_zip_file):
            os.remove(working_zip_file)
        
        with zipfile.ZipFile(working_zip_file, 'w', zipfile.ZIP_DEFLATED) as output_zip:
            output_zip.write(filename, '__main__.py')
            for modulename, module in self.find_libraries(filename):
                for path in module.__path__:
                    for dirname, subdirs, files in os.walk(path):
                        for f in files:
                            # could do some filtering here.
                            filename = os.path.join(dirname, f)
                            zipname  = os.path.join(modulename, dirname[len(path)+1:], f)
                            #print(filename, ' ', zipname)
                            output_zip.write(filename, zipname)
                
        
        with open(output_path, 'wb') as output_file:
            if self.hashbang:
                output_file.write("{}\n".format(self.hashbang).encode('ascii'))
            with open(working_zip_file, 'rb') as prepared_zip: 
                data = prepared_zip.read()
                output_file.write(data)
        
        mode = os.stat(output_path)
        os.chmod(output_path, mode.st_mode | stat.S_IEXEC)
        
        if os.path.exists(working_zip_file):
            os.remove(working_zip_file)
        
        


#!/usr/bin/env python
from distutils.core import setup

setup(
	name="ncdistribute",
	version="1.06",
	description="Distribute python projects as single zip files.",
	author="Nicholas Cole",
	author_email="n@npcole.com",
	url="http://www.npcole.com/",
	packages=['ncdistributerlib'],
    scripts=['ncdistributer.py'],
	license='New BSD License',
	classifiers= [
	    #'Development Status :: 5 - Production/Stable',
	    'Development Status :: 3 - Alpha',
        'Programming Language :: Python :: 3',
	    'Environment :: Console',
	    'Operating System :: POSIX',
	    #'Environment :: Console :: Curses',
	    'Intended Audience :: Developers',
	    'License :: OSI Approved :: BSD License',
	    'Topic :: Terminals'
	    ],
	long_description = """
Provides the script ncdistributer.py that will package up a python script together with any non-standard libraries into a zip file
that can be executed on a computer with a python interpreter installed.

It is similar to tools such as pyzzer, but aims to be even more straight-forward to use.

Unlike projects such as Cython and Nuitka, the this aim of this tool is not to distribute a binary file, but to distribute 
a python package that will work on standard installations of python.

Usage
=====

ncdistributer.py <filenames>...



This is currently an alpha release.  Feedback to nicholas.cole@gmail.com is
welcome.
"""
)



#!/usr/bin/env python3
import sys
import ncdistributerlib
import ncdistributerlib.docopt as docopt

class DistributeApp(ncdistributerlib.Distributor):
    __doc__ = """
{appname} Copyright (c) 2014 Nicholas Cole

This application will package a script for distribution.

Usage:
    {appname} <filename>...
    
    {appname} [-v | -vv] [options] 
              [--exclude-path=<path> --exclude-path=<path>] 
              <filename>...
    
    {appname} -h | --help

Options:
    --hashbang=<string>   The hashbang line to be placed at the 
                          start of the zip file. 
                          [default: #!/usr/bin/env python3]
                         
    --extension=<string>  The extension to be put at the end of
                          the zip archive filename.
                          [default: pyz]
    
    -v | -vv               Verbose


    """.format(appname=sys.argv[0])

    def run(self,):
        for fname in self.arguments['<filename>']:
            self.package(fname)
    
    
if __name__ == "__main__":
    arguments = docopt.docopt(DistributeApp.__doc__)
    A = DistributeApp(exclude_path_list=arguments["--exclude-path"], 
                      hashbang=arguments['--hashbang'],
                      extension=arguments['--extension'],
                      verbose=arguments['-v'])
    A.arguments = arguments
    A.run()
